@@ -17,6 +17,7 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
 
+#include <assert.h>
 #include <errno.h>
 #include <unistd.h>
 
@@ -300,12 +301,14 @@ retval_t ScimBridgeAgentClientListenerImpl::process_message (const ScimBridgeMes
                 key_modifiers |= SCIM_KEY_CapsLockMask;
             } else if (strcmp (modifier_str, SCIM_BRIDGE_MESSAGE_NUM_LOCK) == 0) {
                 key_modifiers |= SCIM_KEY_NumLockMask;
+            } else if (strcmp (modifier_str, SCIM_BRIDGE_MESSAGE_KANA_RO) == 0) {
+                key_modifiers |= SCIM_KEY_QuirkKanaRoMask;
             } else {
                 scim_bridge_perrorln ("Unknown modifier: %s", modifier_str);
             }
         }
 
-        KeyEvent key_event (key_code, key_modifiers);
+        KeyEvent key_event (key_code, key_modifiers, 0);
         return handle_key_event (imcontext_id, key_event);
 
     } else if (strcmp (message_header, SCIM_BRIDGE_MESSAGE_SET_CURSOR_LOCATION) == 0) {
@@ -1041,6 +1044,8 @@ retval_t ScimBridgeAgentClientListenerImpl::forward_key_event (scim_bridge_imcon
     if (key_event.is_super_down ()) ++mod_count;
     if (key_event.is_caps_lock_down ()) ++mod_count;
     if (key_event.is_num_lock_down ()) ++mod_count;
+    
+    if (key_event.mask & SCIM_KEY_QuirkKanaRoMask) ++mod_count;
 
     char *imcontext_id_str;
     scim_bridge_string_from_int (&imcontext_id_str, imcontext_id);
@@ -1090,6 +1095,11 @@ retval_t ScimBridgeAgentClientListenerImpl::forward_key_event (scim_bridge_imcon
         scim_bridge_message_set_argument (message, mod_index, SCIM_BRIDGE_MESSAGE_NUM_LOCK);
         ++mod_index;
     }
+    
+    if (key_event.mask & SCIM_KEY_QuirkKanaRoMask) {
+        scim_bridge_message_set_argument (message, mod_index, SCIM_BRIDGE_MESSAGE_KANA_RO);
+        ++mod_index;
+    }
 
     free (imcontext_id_str);
     free (key_code_str);
