@@ -1207,6 +1207,6 @@ void ScimBridgeAgentIMContextImpl::slot_fallback_commit (IMEngineInstanceBase *i
     if (imcontext == NULL) {
         scim_bridge_pdebugln (8, "No imcontext is connected");
     } else {
-        return imcontext->commit (commit_string);
+        imcontext->commit (commit_string);
     }
 }
