@@ -6,16 +6,15 @@
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.*
+ * License as published by the Free Software Foundation and 
+ * appearing in the file LICENSE.LGPL included in the package of this file.
+ * You can also redistribute it and/or modify it under the terms of 
+ * the GNU General Public License as published by the Free Software Foundation and 
+ * appearing in the file LICENSE.GPL included in the package of this file.
+ *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the
- * GNU Lesser General Public License for more details.*
- * You should have received a copy of the GNU Lesser General Public
- * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
- * Boston, MA  02111-1307  USA
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
 
 #include <assert.h>
@@ -73,10 +72,6 @@ using namespace scim;
 #endif
 
 
-/* Static Constants */
-static const int LOCKFILE_UPDATE_INTERVAL = 60 * 60;
-
-
 /* Class definition */
 class ScimBridgeAgentImpl: public ScimBridgeAgent, public ScimBridgeAgentProtected
 {
@@ -124,11 +119,11 @@ class ScimBridgeAgentImpl: public ScimBridgeAgent, public ScimBridgeAgentProtect
 
     private:
 
+        bool running;
+
         bool noexit_enabled;
         bool standalone_enabled;
 
-        bool active;
-
         list<ScimBridgeAgentSocketClient*> clients;
         size_t client_app_count;
 
@@ -155,31 +150,30 @@ class ScimBridgeAgentImpl: public ScimBridgeAgent, public ScimBridgeAgentProtect
 
         retval_t run_event_loop ();
 
-        retval_t check_lockfile ();
-        retval_t update_lockfile (pid_t pid);
-        retval_t destroy_lockfile ();
+        retval_t check_socket ();
 
         void slot_reload_config (const ConfigPointer &config);
 
 };
 
 /* Helper functions */
-static bool check_socket_frontend ()
+static bool is_socket_frontend_ready ()
 {
     SocketAddress address;
-    SocketClient client;
-
-    uint32 magic;
-
     address.set_address (scim_get_default_socket_frontend_address ());
 
-    if (!client.connect (address))
-        return false;
-
-    if (!scim_socket_open_connection (magic, "ConnectionTester", "SocketFrontEnd", client, 1000)) {
+    SocketClient socket_client;
+    if (!socket_client.connect (address)) {
+        scim_bridge_pdebugln (8, "There is no socket frontend at all.");        
         return false;
     } else {
-        return true;
+        uint32 magic;
+        if (!scim_socket_open_connection (magic, "ConnectionTester", "SocketFrontEnd", socket_client, 1000)) {
+            scim_bridge_pdebugln (8, "Failed to connect the socket fronent.");
+            return false;
+        } else {
+            return true;
+        }
     }
 }
 
@@ -198,7 +192,7 @@ ScimBridgeAgent *ScimBridgeAgent::alloc ()
 
 
 ScimBridgeAgentImpl::ScimBridgeAgentImpl ():
-active (true), noexit_enabled (false), standalone_enabled (false), client_app_count (0),
+running (true), noexit_enabled (false), standalone_enabled (false), client_app_count (0),
 accept_listener (NULL), interruption_listener (NULL), panel_listener (NULL), signal_listener (NULL), display (NULL)
 {
 }
@@ -226,12 +220,7 @@ retval_t ScimBridgeAgentImpl::run_event_loop ()
 {
     scim_bridge_pdebugln (5, "run_event_loop ()");
 
-    timeval lockfile_timestamp;
-    lockfile_timestamp.tv_sec = 0;
-    lockfile_timestamp.tv_usec = 0;
-
-    while (active) {
-
+    while (running) {
         int max_fd = -1;
 
         fd_set read_set;
@@ -240,17 +229,16 @@ retval_t ScimBridgeAgentImpl::run_event_loop ()
         FD_ZERO (&read_set);
         FD_ZERO (&write_set);
         FD_ZERO (&error_set);
-        for (list<ScimBridgeAgentSocketClient*>::iterator i = clients.begin (); i != clients.end ();) {
+        for (list<ScimBridgeAgentSocketClient*>::iterator i = clients.begin (); running && i != clients.end ();) {
             ScimBridgeAgentSocketClient *client = *i;
 
             const scim_bridge_agent_event_type_t triggers = client->get_trigger_events ();
             const int socket_fd = client->get_socket_fd ();
             if (socket_fd < 0) {
-                if (triggers & SCIM_BRIDGE_AGENT_EVENT_ERROR) {
-                    if (!client->handle_event (SCIM_BRIDGE_AGENT_EVENT_ERROR)) {                        
-                        i = clients.erase (i);
-                        delete client;
-                    }
+                if (triggers & SCIM_BRIDGE_AGENT_EVENT_ERROR && !client->handle_event (SCIM_BRIDGE_AGENT_EVENT_ERROR)) {                        
+                    i = clients.erase (i);
+                    delete client;
+                    continue;
                 }
             } else {
                 if (socket_fd > max_fd) max_fd = socket_fd;
@@ -266,28 +254,18 @@ retval_t ScimBridgeAgentImpl::run_event_loop ()
                     FD_SET (socket_fd, &error_set);
                     scim_bridge_pdebugln (1, "FD (%d) is registred as a error socket", socket_fd);
                 }
-                ++i;
             }
-        }
-
-        timeval current_time;
-        gettimeofday (&current_time, NULL);
-
-        int past_sec = (current_time.tv_sec - lockfile_timestamp.tv_sec) + (current_time.tv_usec - lockfile_timestamp.tv_usec) / 1000000;
-        if (past_sec > LOCKFILE_UPDATE_INTERVAL) {
-            update_lockfile (getpid ());
-            lockfile_timestamp = current_time;
-            past_sec = 0;
+            ++i;
         }
 
         scim_bridge_pdebugln (2, "Waiting for an event...");
-        timeval remain_time;
-        remain_time.tv_sec = LOCKFILE_UPDATE_INTERVAL - past_sec;
-        remain_time.tv_usec = 0;
-        const int retval = select (max_fd + 1, &read_set, &write_set, &error_set, &remain_time);
+        const int retval = select (max_fd + 1, &read_set, &write_set, &error_set, NULL);
         if (retval < 0 && errno != EINTR) {
-            scim_bridge_perrorln ("An exception occurred at selecting the sockets: %s", strerror (errno));
-            abort ();
+            if (errno == EINTR || errno == EAGAIN || errno == EWOULDBLOCK) {
+                continue;
+            } else {
+                scim_bridge_perrorln ("An exception occurred at selecting the sockets: %s", strerror (errno));
+            }
         }
 
         const bool interrupted = interruption_listener->is_interrupted ();
@@ -302,7 +280,7 @@ retval_t ScimBridgeAgentImpl::run_event_loop ()
 
             scim_bridge_agent_event_type_t events = SCIM_BRIDGE_AGENT_EVENT_NONE;
 
-            if (FD_ISSET (socket_fd, &read_set) && (triggers & SCIM_BRIDGE_AGENT_EVENT_READ)) {
+            if (socket_fd >= 0 && FD_ISSET (socket_fd, &read_set) && (triggers & SCIM_BRIDGE_AGENT_EVENT_READ)) {
                 if (events == SCIM_BRIDGE_AGENT_EVENT_NONE) {
                     scim_bridge_pdebug (2, "Invoked triggers: READ");
                 } else {
@@ -310,7 +288,7 @@ retval_t ScimBridgeAgentImpl::run_event_loop ()
                 }
                 events |= SCIM_BRIDGE_AGENT_EVENT_READ;
             }
-            if (FD_ISSET (socket_fd, &write_set) && (triggers & SCIM_BRIDGE_AGENT_EVENT_WRITE)) {
+            if (socket_fd >= 0 && FD_ISSET (socket_fd, &write_set) && (triggers & SCIM_BRIDGE_AGENT_EVENT_WRITE)) {
                 if (events == SCIM_BRIDGE_AGENT_EVENT_NONE) {
                     scim_bridge_pdebug (2, "Invoked triggers: WRITE");
                 } else {
@@ -318,7 +296,7 @@ retval_t ScimBridgeAgentImpl::run_event_loop ()
                 }
                 events |= SCIM_BRIDGE_AGENT_EVENT_WRITE;
             }
-            if (FD_ISSET (socket_fd, &error_set) && (triggers & SCIM_BRIDGE_AGENT_EVENT_ERROR)) {
+            if ((socket_fd < 0 || FD_ISSET (socket_fd, &error_set)) && (triggers & SCIM_BRIDGE_AGENT_EVENT_ERROR)) {
                 if (events == SCIM_BRIDGE_AGENT_EVENT_NONE) {
                     scim_bridge_pdebug (2, "Invoked triggers: ERROR");
                 } else {
@@ -363,16 +341,10 @@ retval_t ScimBridgeAgentImpl::launch ()
             scim_bridge_perrorln ("Cannot fork myself: %s", strerror (errno));
             return RETVAL_FAILED;
         } else if (pid > 0) {
-            // This is the parent process.
-            if (update_lockfile (pid)) {
-                scim_bridge_perrorln ("The cannot create the lockfile...");
-                return RETVAL_FAILED;
-            }
-
             _exit (0);
         } else {
             // This is the child process.
-            if (chdir ("/")) {
+            if (chdir ("/tmp")) {
                 scim_bridge_perrorln ("Cannot change the working directory: %s", strerror (errno));
                 return RETVAL_FAILED;
             }
@@ -382,91 +354,25 @@ retval_t ScimBridgeAgentImpl::launch ()
         }
 
         scim_bridge_pdebugln (5, "Daemonize done");
-
-    } else {
-
-        if (update_lockfile (getpid ())) {
-            scim_bridge_perrorln ("Cannot create the lockfile...");
-            return RETVAL_FAILED;
-        }
-
     }
 
     run_event_loop ();
-    destroy_lockfile ();
     return RETVAL_SUCCEEDED;
 }
 
 
-retval_t ScimBridgeAgentImpl::check_lockfile ()
-{
-    scim_bridge_pdebugln (8, "Checking the lockfile...");
-    const char *lockfile_path = scim_bridge_path_get_lockfile ();
-    ifstream input_stream (lockfile_path);
-
-    pid_t pid = -1;
-    input_stream >> pid;
-
-    input_stream.close ();
-
-    if (input_stream.good () && pid > 0) {
-        scim_bridge_pdebugln (5, "Pinging for the process: pid = %d", pid);
-        if (kill (pid, 0) == 0) {
-            scim_bridge_println ("Another agent is running...");
-            return RETVAL_FAILED;
-        }
-    }
-
-    return RETVAL_SUCCEEDED;
-}
-
-
-retval_t ScimBridgeAgentImpl::update_lockfile (pid_t pid)
-{
-    scim_bridge_pdebugln (2, "Updating the lockfile...");
-
-    const char *lockfile_path = scim_bridge_path_get_lockfile ();
-    ofstream output_stream (lockfile_path);
-
-    output_stream << pid << endl;
-    output_stream.close ();
-
-    if (!output_stream.good ()) {
-        scim_bridge_perrorln ("Cannot update the lockfile");
-        return RETVAL_FAILED;
-    } else {
-        scim_bridge_pdebugln (7, "The lockfile is updated.");
-        return RETVAL_SUCCEEDED;
-    }
-}
-
-
-retval_t ScimBridgeAgentImpl::destroy_lockfile ()
-{
-    scim_bridge_pdebugln (7, "Destroying the lockfile...");
-
-    const char *lockfile_path = scim_bridge_path_get_lockfile ();
-
-    if (unlink (lockfile_path)) {
-        scim_bridge_perrorln ("Cannot destroy the lockfile");
-        return RETVAL_FAILED;
-    } else {
-        scim_bridge_perrorln ("The lockfile is destroied");
-        return RETVAL_SUCCEEDED;
-    }
-}
-
-
 retval_t ScimBridgeAgentImpl::initialize ()
 {
+    accept_listener = ScimBridgeAgentAcceptListener::alloc (this);
+    if (accept_listener == NULL) return RETVAL_FAILED;
+    clients.push_back (accept_listener);
+    
     display = scim_bridge_alloc_display ();
     if (scim_bridge_display_fetch_current (display)) {
         scim_bridge_perrorln ("Failed to allocate the current display");
         return RETVAL_FAILED;
     }
 
-    if (check_lockfile ()) return RETVAL_FAILED;
-
     if (initialize_scim ()) {
         scim_bridge_perrorln ("Failed to initialize scim");
         return RETVAL_FAILED;
@@ -484,10 +390,6 @@ retval_t ScimBridgeAgentImpl::initialize ()
     if (panel_listener == NULL) return RETVAL_FAILED;
     clients.push_back (panel_listener);
 
-    accept_listener = ScimBridgeAgentAcceptListener::alloc (this);
-    if (accept_listener == NULL) return RETVAL_FAILED;
-    clients.push_back (accept_listener);
-
     signal_listener = ScimBridgeAgentSignalListener::alloc (this);
     if (signal_listener == NULL) return RETVAL_FAILED;
     clients.push_back (signal_listener);
@@ -529,113 +431,57 @@ retval_t ScimBridgeAgentImpl::initialize_scim ()
     // Get system language.
     scim_language = scim_get_locale_language (scim_get_current_locale ());
 
-    // Get modules list
-    vector<String> imengine_vector;
-    vector<String> config_vector;
-    scim_get_imengine_module_list (imengine_vector);
-    scim_get_config_module_list (config_vector);
+    // Get imengine modules
+    vector<String> imengine_module_names;
+    scim_get_imengine_module_list (imengine_module_names);
+    if (find (imengine_module_names.begin (), imengine_module_names.end (), "socket") == imengine_module_names.end ()) {
+        scim_bridge_perrorln ("There is no socket frontend of IMEngines for SCIM...");
+        return RETVAL_FAILED;
+    }
 
-    // Use socket if avaiable.
-    bool use_socket = true;
-    if (find (imengine_vector.begin (), imengine_vector.end (), "socket") == imengine_vector.end () ||
-        find (config_vector.begin (), config_vector.end (), "socket") == config_vector.end ())
-        use_socket = false;
+    // Get config modules
+    vector<String> config_module_names;
+    scim_get_config_module_list (config_module_names);
 
-    // Use simple config module as default if available.
-    String config_module_name;
-    if (config_vector.size () > 0) {
-        config_module_name = scim_global_config_read (SCIM_GLOBAL_CONFIG_DEFAULT_CONFIG_MODULE, String ("simple"));
-        if (find (config_vector.begin (), config_vector.end (), config_module_name) == config_vector.end ())
-            config_module_name = config_vector [0];
+    if (find (config_module_names.begin (), config_module_names.end (), "socket") == config_module_names.end ()) {
+        scim_bridge_perrorln ("There is no socket frontend of config module for SCIM...");
+        return RETVAL_FAILED;
+    }
+
+    // If there is no socket frontend running, launch one.
+    if (!is_socket_frontend_ready ()) {
+        scim_bridge_pdebugln (8, "Launching a SCIM daemon with Socket FrontEnd...");
+        const String server_config_module_name = scim_global_config_read (SCIM_GLOBAL_CONFIG_DEFAULT_CONFIG_MODULE, String ("simple"));
+        char *new_argv [] = {"--no-stay","-d", NULL};
+        scim_launch (true, server_config_module_name.c_str (), "all", "socket", new_argv);
+
+        // Wait until the connection is established.
+        for (int i = 0; i < 100; ++i) {
+            if (is_socket_frontend_ready ()) {
+                break;
+            } else if (i < 99) {
+                usleep (100000);
+            } else {
+                scim_bridge_perrorln ("Cannot establish the socket connection...");
+                return RETVAL_FAILED;
+            }
+        }
+    }
+
+    // load config module
+    scim_bridge_pdebugln (2, "Loading Config module...: socket");
+    scim_config_module = new ConfigModule ("socket");
+
+    //create config instance
+    if (scim_config_module != NULL && scim_config_module->valid ()) {
+        scim_config = scim_config_module->create_config ();
     } else {
-        config_module_name = "dummy";
-    }
-
-    // Use user defined immodules if available
-    bool use_user_imegine = false;
-    vector <String> user_imengine_vector;
-    const char *imengine_list_str = getenv ("SCIM_BRIDGE_IMENGINE_MODULES");
-    if (imengine_list_str != NULL) {
-        vector <String> spec_imengine_vector;
-        scim_split_string_list (spec_imengine_vector, imengine_list_str, ',');
-
-        user_imengine_vector.clear ();
-
-        for (size_t i = 0; i < spec_imengine_vector.size (); ++i) {
-            if (find (imengine_vector.begin (), imengine_vector.end (), imengine_vector [i]) != imengine_vector.end ()) {
-                user_imengine_vector.push_back (spec_imengine_vector [i]);
-                use_user_imegine = true;
-            }
-        }
-    }
-
-    // If you try to use the socket feature manually,
-    // then let you do it by yourself.
-    if (config_module_name == "socket" ||
-        find (user_imengine_vector.begin (), user_imengine_vector.end (), "socket") != user_imengine_vector.end ())
-        use_socket = false;
-
-    // Try to start a SCIM SocketFrontEnd process if necessary.
-    if (scim_get_default_socket_frontend_address () != scim_get_default_socket_imengine_address () &&
-        scim_get_default_socket_frontend_address () != scim_get_default_socket_config_address ())
-        use_socket = false;
-
-    if (use_socket) {
-        // If no Socket FrontEnd is running, then launch one.
-        // And set manual to false.
-        if (!check_socket_frontend ()) {
-            scim_bridge_perrorln ("Launching a SCIM daemon with Socket FrontEnd...");
-            char *new_argv [] = { "--no-stay", 0 };
-            scim_launch (true,
-                config_module_name,
-                (!user_imengine_vector.empty () ? scim_combine_string_list (user_imengine_vector, ',') : "all"),
-                "socket",
-                new_argv);
-            use_user_imegine = false;
-        }
-
-        // If there is one Socket FrontEnd running and it's not manual mode,
-        // then just use this Socket Frontend.
-        if (!use_user_imegine) {
-            for (int i = 0; i < 100; ++i) {
-                if (check_socket_frontend ()) {
-                    config_module_name = "socket";
-                    user_imengine_vector.clear ();
-                    user_imengine_vector.push_back ("socket");
-                    break;
-                }
-                scim_usleep (100000);
-            }
-        }
-    }
-
-    if (config_module_name != "dummy") {
-        //load config module
-        scim_bridge_pdebugln (2, "Loading Config module...: %s", config_module_name.c_str ());
-        scim_config_module = new ConfigModule (config_module_name);
-
-        //create config instance
-        if (scim_config_module != NULL && scim_config_module->valid ())
-            scim_config = scim_config_module->create_config ();
-    }
-
-    if (scim_config.null ()) {
-        scim_bridge_pdebugln (2, "Config module cannot be loaded, using dummy Config.");
-
-        if (scim_config_module) delete scim_config_module;
-        scim_config_module = NULL;
-
-        scim_config = new DummyConfig ();
-        config_module_name = "dummy";
+        scim_bridge_pdebugln (2, "Cannot load the socket config module...");
+        return RETVAL_FAILED;
     }
 
     // create backend
-    scim_backend = new CommonBackEnd (scim_config, !user_imengine_vector.empty () ? user_imengine_vector : imengine_vector);
-
-    if (scim_backend.null ()) {
-        scim_bridge_perrorln ("Cannot create BackEnd Object!");
-        return RETVAL_FAILED;
-    }
+    scim_backend = new CommonBackEnd (scim_config, imengine_module_names);
 
     scim_bridge_pdebugln (4, "Initialize scim, done!");
     return RETVAL_SUCCEEDED;
@@ -645,6 +491,14 @@ retval_t ScimBridgeAgentImpl::initialize_scim ()
 retval_t ScimBridgeAgentImpl::finalize_scim ()
 {
     scim_bridge_pdebugln (6, "Finalizing scim...");
+    
+    scim_backend.reset ();
+    scim_config.reset ();
+
+    if (scim_config_module != NULL) {
+        delete scim_config_module;
+        scim_config_module = NULL;
+    }
 
     return RETVAL_SUCCEEDED;
 }
@@ -683,7 +537,7 @@ void ScimBridgeAgentImpl::quit ()
 {
     scim_bridge_pdebugln (5, "quit ()");
 
-    active = false;
+    running = false;
 }
 
 
@@ -812,6 +666,9 @@ bool ScimBridgeAgentImpl::filter_hotkeys (scim_bridge_imcontext_id_t imcontext_i
 bool ScimBridgeAgentImpl::filter_key_event (scim_bridge_imcontext_id_t imcontext_id, const KeyEvent &key_event)
 {
     scim_bridge_pdebugln (5, "filter_key_event ()");
+    
+    KeyEvent new_key_event = key_event;
+    new_key_event.layout = scim_keyboard_layout;
 
     ScimBridgeAgentIMContext *imcontext = ScimBridgeAgentIMContext::find (imcontext_id);
     if (imcontext == NULL) {
@@ -820,9 +677,9 @@ bool ScimBridgeAgentImpl::filter_key_event (scim_bridge_imcontext_id_t imcontext
     } else {
         bool consumed = false;
         panel_listener->prepare (imcontext_id);
-        if (filter_hotkeys (imcontext_id, key_event)) {
+        if (filter_hotkeys (imcontext_id, new_key_event)) {
             consumed = true;
-        } else if (imcontext->filter_key_event (key_event)) {
+        } else if (imcontext->filter_key_event (new_key_event)) {
             consumed = true;
         } else {
             consumed = false;
@@ -884,10 +741,9 @@ void ScimBridgeAgentImpl::request_factory_menu ()
     scim_bridge_pdebugln (6, "request_factory_menu ()");
 
     vector<IMEngineFactoryPointer> factories;
-    vector<PanelFactoryInfo> menu;
-
     scim_backend->get_factories_for_encoding (factories, "UTF-8");
 
+    vector<PanelFactoryInfo> menu;
     for (size_t i = 0; i < factories.size (); ++i) {
         menu.push_back (PanelFactoryInfo (factories [i]->get_uuid (), utf8_wcstombs (factories [i]->get_name ()), factories [i]->get_language (), factories [i]->get_icon_file ()));
     }
@@ -907,6 +763,5 @@ void ScimBridgeAgentImpl::save_config ()
 {
     scim_bridge_pdebugln (6, "save_config ()");
 
-    scim_config->write (String (SCIM_CONFIG_FRONTEND_IM_OPENED_BY_DEFAULT),
-        ScimBridgeAgentIMContext::is_enabled_by_default ());
+    scim_config->write (String (SCIM_CONFIG_FRONTEND_IM_OPENED_BY_DEFAULT), ScimBridgeAgentIMContext::is_enabled_by_default ());
 }
