@@ -602,6 +602,8 @@ static retval_t received_message_forward_key_event (const ScimBridgeMessage *mes
     boolean num_lock_down = FALSE;
     boolean unknown_down = FALSE;
 
+    boolean kana_ro = FALSE;
+
     int i;
     for (i = 0; i < modifier_count; ++i) {
         const char *modifier_str = scim_bridge_message_get_argument (message, i + 3);
@@ -622,93 +624,116 @@ static retval_t received_message_forward_key_event (const ScimBridgeMessage *mes
             caps_lock_down = TRUE;
         } else if (strcmp (modifier_str, SCIM_BRIDGE_MESSAGE_NUM_LOCK) == 0) {
             num_lock_down = TRUE;
+        } else if (strcmp (modifier_str, SCIM_BRIDGE_MESSAGE_KANA_RO) == 0) {
+            kana_ro = TRUE;
         } else {
             scim_bridge_perrorln ("Unknown modifier: %s", modifier_str);
             unknown_down = TRUE;
         }
     }
 
-    char *modifiers_str = alloca (sizeof (char) * (strlen (SCIM_BRIDGE_MESSAGE_SHIFT) + strlen (SCIM_BRIDGE_MESSAGE_CONTROL) + strlen (SCIM_BRIDGE_MESSAGE_ALT) + strlen (SCIM_BRIDGE_MESSAGE_META) + strlen (SCIM_BRIDGE_MESSAGE_SUPER) + strlen (SCIM_BRIDGE_MESSAGE_HYPER) + strlen (SCIM_BRIDGE_MESSAGE_CAPS_LOCK) + strlen (SCIM_BRIDGE_MESSAGE_NUM_LOCK) + strlen (SCIM_BRIDGE_MESSAGE_UNKNOWN) + 9));
-    size_t modifier_str_index = 0;
-
-    modifiers_str[0] = '\0';
-
-    if (shift_down) {
-        strcpy (modifiers_str + modifier_str_index, SCIM_BRIDGE_MESSAGE_SHIFT);
-        modifier_str_index += strlen (SCIM_BRIDGE_MESSAGE_SHIFT);
-    }
-    if (control_down) {
-        if (modifier_str_index > 0) {
-            strcpy (modifiers_str + modifier_str_index, "+");
-            modifier_str_index += strlen ("+");
-        }
-        strcpy (modifiers_str + modifier_str_index, SCIM_BRIDGE_MESSAGE_CONTROL);
-        modifier_str_index += strlen (SCIM_BRIDGE_MESSAGE_CONTROL);
-    }
-    if (alt_down) {
-        if (modifier_str_index > 0) {
-            strcpy (modifiers_str + modifier_str_index, "+");
-            modifier_str_index += strlen ("+");
-        }
-        strcpy (modifiers_str + modifier_str_index, SCIM_BRIDGE_MESSAGE_ALT);
-        modifier_str_index += strlen (SCIM_BRIDGE_MESSAGE_ALT);
-    }
-    if (meta_down) {
-        if (modifier_str_index > 0) {
-            strcpy (modifiers_str + modifier_str_index, "+");
-            modifier_str_index += strlen ("+");
-        }
-        strcpy (modifiers_str + modifier_str_index, SCIM_BRIDGE_MESSAGE_META);
-        modifier_str_index += strlen (SCIM_BRIDGE_MESSAGE_META);
-    }
-    if (super_down) {
-        if (modifier_str_index > 0) {
-            strcpy (modifiers_str + modifier_str_index, "+");
-            modifier_str_index += strlen ("+");
-        }
-        strcpy (modifiers_str + modifier_str_index, SCIM_BRIDGE_MESSAGE_SUPER);
-        modifier_str_index += strlen (SCIM_BRIDGE_MESSAGE_SUPER);
-    }
-    if (hyper_down) {
-        if (modifier_str_index > 0) {
-            strcpy (modifiers_str + modifier_str_index, "+");
-            modifier_str_index += strlen ("+");
-        }
-        strcpy (modifiers_str + modifier_str_index, SCIM_BRIDGE_MESSAGE_HYPER);
-        modifier_str_index += strlen (SCIM_BRIDGE_MESSAGE_HYPER);
-    }
-    if (caps_lock_down) {
-        if (modifier_str_index > 0) {
-            strcpy (modifiers_str + modifier_str_index, "+");
-            modifier_str_index += strlen ("+");
-        }
-        strcpy (modifiers_str + modifier_str_index, SCIM_BRIDGE_MESSAGE_CAPS_LOCK);
-        modifier_str_index += strlen (SCIM_BRIDGE_MESSAGE_CAPS_LOCK);
-    }
-    if (num_lock_down) {
-        if (modifier_str_index > 0) {
-            strcpy (modifiers_str + modifier_str_index, "+");
-            modifier_str_index += strlen ("+");
-        }
-        strcpy (modifiers_str + modifier_str_index, SCIM_BRIDGE_MESSAGE_NUM_LOCK);
-        modifier_str_index += strlen (SCIM_BRIDGE_MESSAGE_NUM_LOCK);
-    }
-    if (unknown_down) {
-        if (modifier_str_index > 0) {
-            strcpy (modifiers_str + modifier_str_index, "+");
-            modifier_str_index += strlen ("+");
-        }
-        strcpy (modifiers_str + modifier_str_index, SCIM_BRIDGE_MESSAGE_UNKNOWN);
-        modifier_str_index += strlen (SCIM_BRIDGE_MESSAGE_UNKNOWN);
-    }
-
     scim_bridge_imcontext_id_t ic_id;
     unsigned int key_code;
     boolean key_pressed;
     if (scim_bridge_string_to_int (&ic_id, ic_id_str) || scim_bridge_string_to_uint (&key_code, key_code_str)
         || scim_bridge_string_to_boolean (&key_pressed, key_pressed_str)) {
 
-        scim_bridge_perrorln ("Invalid arguments for the message: %s (%s, %s, %s, %s)", header, ic_id_str, key_code_str, key_pressed_str, modifiers_str);
+        scim_bridge_perror ("Invalid arguments for the message: %s (%s, %s, %s", header, ic_id_str, key_code_str, key_pressed_str);
+    
+        boolean first_modifier = TRUE;
+        
+        if (shift_down) {
+            if (first_modifier) {
+                scim_bridge_perror (", ");
+                first_modifier = FALSE;
+            } else {
+                scim_bridge_perror (" + ");
+            }
+            scim_bridge_perror ("%s", SCIM_BRIDGE_MESSAGE_SHIFT);
+        }
+        if (control_down) {
+            if (first_modifier) {
+                scim_bridge_perror (", ");
+                first_modifier = FALSE;
+            } else {
+                scim_bridge_perror (" + ");
+            }
+            scim_bridge_perror ("%s", SCIM_BRIDGE_MESSAGE_CONTROL);
+        }
+        if (alt_down) {
+            if (first_modifier) {
+                scim_bridge_perror (", ");
+                first_modifier = FALSE;
+            } else {
+                scim_bridge_perror (" + ");
+            }
+            scim_bridge_perror ("%s", SCIM_BRIDGE_MESSAGE_ALT);
+        }
+        if (meta_down) {
+            if (first_modifier) {
+                scim_bridge_perror (", ");
+                first_modifier = FALSE;
+            } else {
+                scim_bridge_perror (" + ");
+            }
+            scim_bridge_perror ("%s", SCIM_BRIDGE_MESSAGE_META);
+        }
+        if (super_down) {
+            if (first_modifier) {
+                scim_bridge_perror (", ");
+                first_modifier = FALSE;
+            } else {
+                scim_bridge_perror (" + ");
+            }
+            scim_bridge_perror ("%s", SCIM_BRIDGE_MESSAGE_SUPER);
+        }
+        if (hyper_down) {
+            if (first_modifier) {
+                scim_bridge_perror (", ");
+                first_modifier = FALSE;
+            } else {
+                scim_bridge_perror (" + ");
+            }
+            scim_bridge_perror ("%s", SCIM_BRIDGE_MESSAGE_HYPER);
+        }
+        if (caps_lock_down) {
+            if (first_modifier) {
+                scim_bridge_perror (", ");
+                first_modifier = FALSE;
+            } else {
+                scim_bridge_perror (" + ");
+            }
+            scim_bridge_perror ("%s", SCIM_BRIDGE_MESSAGE_CAPS_LOCK);
+        }
+        if (num_lock_down) {
+            if (first_modifier) {
+                scim_bridge_perror (", ");
+                first_modifier = FALSE;
+            } else {
+                scim_bridge_perror (" + ");
+            }
+            scim_bridge_perror ("%s", SCIM_BRIDGE_MESSAGE_NUM_LOCK);
+        }
+        if (kana_ro) {
+            if (first_modifier) {
+                scim_bridge_perror (", ");
+                first_modifier = FALSE;
+            } else {
+                scim_bridge_perror (" + ");
+            }
+            scim_bridge_perror ("%s", SCIM_BRIDGE_MESSAGE_KANA_RO);
+        }
+        if (unknown_down) {
+            if (first_modifier) {
+                scim_bridge_perror (", ");
+                first_modifier = FALSE;
+            } else {
+                scim_bridge_perror (" + ");
+            }
+            scim_bridge_perror ("%s", SCIM_BRIDGE_MESSAGE_UNKNOWN);
+        }
+        scim_bridge_perrorln (")");
+        
     } else {
         ScimBridgeClientIMContext *imcontext = scim_bridge_client_find_imcontext (ic_id);
 
@@ -724,6 +749,8 @@ static retval_t received_message_forward_key_event (const ScimBridgeMessage *mes
             scim_bridge_key_event_set_hyper_down (key_event, hyper_down);
             scim_bridge_key_event_set_caps_lock_down (key_event, caps_lock_down);
             scim_bridge_key_event_set_num_lock_down (key_event, num_lock_down);
+            
+            scim_bridge_key_event_set_quirk_enabled (key_event, SCIM_BRIDGE_KEY_QUIRK_KANA_RO, kana_ro);
 
             scim_bridge_client_imcontext_forward_key_event (imcontext, key_event);
             scim_bridge_free_key_event (key_event);
@@ -1496,84 +1523,94 @@ retval_t scim_bridge_client_handle_key_event (const ScimBridgeClientIMContext *i
 
     const scim_bridge_key_code_t key_code = scim_bridge_key_event_get_code (key_event);
 
-    char *modifiers_str = alloca (sizeof (char) * (strlen (SCIM_BRIDGE_MESSAGE_SHIFT) + strlen (SCIM_BRIDGE_MESSAGE_CONTROL) + strlen (SCIM_BRIDGE_MESSAGE_ALT) + strlen (SCIM_BRIDGE_MESSAGE_META) + strlen (SCIM_BRIDGE_MESSAGE_META) + strlen (SCIM_BRIDGE_MESSAGE_SUPER) + strlen (SCIM_BRIDGE_MESSAGE_HYPER) + strlen (SCIM_BRIDGE_MESSAGE_CAPS_LOCK) + strlen (SCIM_BRIDGE_MESSAGE_NUM_LOCK) + strlen (SCIM_BRIDGE_MESSAGE_UNKNOWN) + 9));
-    size_t modifier_str_index = 0;
-    size_t modifier_count = 0;
+    const boolean key_pressed = scim_bridge_key_event_is_pressed (key_event);
+    scim_bridge_pdebug (5, "scim_bridge_client_key_event_occurred: ic = %d, key_code = %u, pressed = %s", id, key_code, key_pressed ? "true":"false");
+
+    int modifier_count = 0;
 
-    modifiers_str[0] = '\0';
     if (scim_bridge_key_event_is_shift_down (key_event)) {
-        strcpy (modifiers_str + modifier_str_index, SCIM_BRIDGE_MESSAGE_SHIFT);
-        modifier_str_index += strlen (SCIM_BRIDGE_MESSAGE_SHIFT);
+        if (modifier_count == 0) {
+            scim_bridge_pdebug (5, ", modifier = ");
+        } else {
+            scim_bridge_pdebug (5, " + ");
+        }
         ++modifier_count;
+        scim_bridge_pdebug (5, "%s", SCIM_BRIDGE_MESSAGE_SHIFT);
     }
     if (scim_bridge_key_event_is_control_down (key_event)) {
-        if (modifier_str_index > 0) {
-            strcpy (modifiers_str + modifier_str_index, "+");
-            modifier_str_index += strlen ("+");
+        if (modifier_count == 0) {
+            scim_bridge_pdebug (5, ", modifier = ");
+        } else {
+            scim_bridge_pdebug (5, " + ");
         }
-        strcpy (modifiers_str + modifier_str_index, SCIM_BRIDGE_MESSAGE_CONTROL);
-        modifier_str_index += strlen (SCIM_BRIDGE_MESSAGE_CONTROL);
         ++modifier_count;
+        scim_bridge_pdebug (5, "%s", SCIM_BRIDGE_MESSAGE_CONTROL);
     }
     if (scim_bridge_key_event_is_alt_down (key_event)) {
-        if (modifier_str_index > 0) {
-            strcpy (modifiers_str + modifier_str_index, "+");
-            modifier_str_index += strlen ("+");
+        if (modifier_count == 0) {
+            scim_bridge_pdebug (5, ", modifier = ");
+        } else {
+            scim_bridge_pdebug (5, " + ");
         }
-        strcpy (modifiers_str + modifier_str_index, SCIM_BRIDGE_MESSAGE_ALT);
-        modifier_str_index += strlen (SCIM_BRIDGE_MESSAGE_ALT);
         ++modifier_count;
+        scim_bridge_pdebug (5, "%s", SCIM_BRIDGE_MESSAGE_ALT);
     }
     if (scim_bridge_key_event_is_meta_down (key_event)) {
-        if (modifier_str_index > 0) {
-            strcpy (modifiers_str + modifier_str_index, "+");
-            modifier_str_index += strlen ("+");
+        if (modifier_count == 0) {
+            scim_bridge_pdebug (5, ", modifier = ");
+        } else {
+            scim_bridge_pdebug (5, " + ");
         }
-        strcpy (modifiers_str + modifier_str_index, SCIM_BRIDGE_MESSAGE_META);
-        modifier_str_index += strlen (SCIM_BRIDGE_MESSAGE_META);
         ++modifier_count;
+        scim_bridge_pdebug (5, "%s", SCIM_BRIDGE_MESSAGE_META);
     }
     if (scim_bridge_key_event_is_super_down (key_event)) {
-        if (modifier_str_index > 0) {
-            strcpy (modifiers_str + modifier_str_index, "+");
-            modifier_str_index += strlen ("+");
+        if (modifier_count == 0) {
+            scim_bridge_pdebug (5, ", modifier = ");
         } else {
-            strcpy (modifiers_str + modifier_str_index, SCIM_BRIDGE_MESSAGE_SUPER);
-            modifier_str_index += strlen (SCIM_BRIDGE_MESSAGE_SUPER);
-            ++modifier_count;
+            scim_bridge_pdebug (5, " + ");
         }
+        ++modifier_count;
+        scim_bridge_pdebug (5, "%s", SCIM_BRIDGE_MESSAGE_SUPER);
     }
     if (scim_bridge_key_event_is_hyper_down (key_event)) {
-        if (modifier_str_index > 0) {
-            strcpy (modifiers_str + modifier_str_index, "+");
-            modifier_str_index += strlen ("+");
+        if (modifier_count == 0) {
+            scim_bridge_pdebug (5, ", modifier = ");
+        } else {
+            scim_bridge_pdebug (5, " + ");
         }
-        strcpy (modifiers_str + modifier_str_index, SCIM_BRIDGE_MESSAGE_HYPER);
-        modifier_str_index += strlen (SCIM_BRIDGE_MESSAGE_HYPER);
         ++modifier_count;
+        scim_bridge_pdebug (5, "%s", SCIM_BRIDGE_MESSAGE_HYPER);
     }
     if (scim_bridge_key_event_is_caps_lock_down (key_event)) {
-        if (modifier_str_index > 0) {
-            strcpy (modifiers_str + modifier_str_index, "+");
-            modifier_str_index += strlen ("+");
+        if (modifier_count == 0) {
+            scim_bridge_pdebug (5, ", modifier = ");
+        } else {
+            scim_bridge_pdebug (5, " + ");
         }
-        strcpy (modifiers_str + modifier_str_index, SCIM_BRIDGE_MESSAGE_CAPS_LOCK);
-        modifier_str_index += strlen (SCIM_BRIDGE_MESSAGE_CAPS_LOCK);
         ++modifier_count;
+        scim_bridge_pdebug (5, "%s", SCIM_BRIDGE_MESSAGE_CAPS_LOCK);
     }
     if (scim_bridge_key_event_is_num_lock_down (key_event)) {
-        if (modifier_str_index > 0) {
-            strcpy (modifiers_str + modifier_str_index, "+");
-            modifier_str_index += strlen ("+");
+        if (modifier_count == 0) {
+            scim_bridge_pdebug (5, ", modifier = ");
+        } else {
+            scim_bridge_pdebug (5, " + ");
         }
-        strcpy (modifiers_str + modifier_str_index, SCIM_BRIDGE_MESSAGE_NUM_LOCK);
-        modifier_str_index += strlen (SCIM_BRIDGE_MESSAGE_NUM_LOCK);
         ++modifier_count;
+        scim_bridge_pdebug (5, "%s", SCIM_BRIDGE_MESSAGE_NUM_LOCK);
     }
-
-    const boolean key_pressed = scim_bridge_key_event_is_pressed (key_event);
-    scim_bridge_pdebugln (5, "scim_bridge_client_key_event_occurred: ic = %d, key_code = %u, pressed = %s, modifiers = %s", id, key_code, key_pressed ? "true":"false", modifiers_str);
-
+    if (scim_bridge_key_event_is_quirk_enabled (key_event, SCIM_BRIDGE_KEY_QUIRK_KANA_RO)) {
+        if (modifier_count == 0) {
+            scim_bridge_pdebug (5, ", modifier = ");
+        } else {
+            scim_bridge_pdebug (5, " + ");
+        }
+        ++modifier_count;
+        scim_bridge_pdebug (5, "%s", SCIM_BRIDGE_MESSAGE_KANA_RO);
+    }
+    scim_bridge_pdebugln (5, "");
+    
     scim_bridge_pdebugln (5, "Sending 'handle_key_event' message: ic_id = %d", id);
 
     ScimBridgeMessage *message = scim_bridge_alloc_message (SCIM_BRIDGE_MESSAGE_HANDLE_KEY_EVENT, modifier_count + 3);
@@ -1628,6 +1665,10 @@ retval_t scim_bridge_client_handle_key_event (const ScimBridgeClientIMContext *i
         scim_bridge_message_set_argument (message, arg_index, SCIM_BRIDGE_MESSAGE_NUM_LOCK);
         ++arg_index;
     }
+    if (scim_bridge_key_event_is_quirk_enabled (key_event, SCIM_BRIDGE_KEY_QUIRK_KANA_RO)) {
+        scim_bridge_message_set_argument (message, arg_index, SCIM_BRIDGE_MESSAGE_KANA_RO);
+        ++arg_index;
+    }
 
     pending_response.header = SCIM_BRIDGE_MESSAGE_KEY_EVENT_HANDLED;
     pending_response.consumed = FALSE;
