@@ -346,6 +346,17 @@ ScimBridgeKeyEvent* scim_bridge_key_event_x11_to_bridge (const XEvent *x_event)
         scim_bridge_key_event_set_meta_down (bridge_key_event, TRUE);
     }
 
+    if (scim_bridge_key_event_get_code (bridge_key_event) == SCIM_BRIDGE_KEY_CODE_backslash) {
+        boolean kana_ro = FALSE;
+        int keysym_size = 0;
+        KeySym *keysyms = XGetKeyboardMapping (x_key_event->display, x_key_event->keycode, 1, &keysym_size);
+        if (keysyms != NULL) {
+            kana_ro = (keysyms[0] == XK_backslash && keysyms[1] == XK_underscore);
+            XFree (keysyms);
+        }
+        scim_bridge_key_event_set_quirk_enabled (bridge_key_event, SCIM_BRIDGE_KEY_QUIRK_KANA_RO, kana_ro);
+    }
+    
     return bridge_key_event;
 }
 
