@@ -243,6 +243,11 @@ static const char SCIM_BRIDGE_MESSAGE_CAPS_LOCK[] = "caps_lock";
  */
 static const char SCIM_BRIDGE_MESSAGE_NUM_LOCK[] = "num_lock";
 
+/**
+ * The string constant of "kana_ro" for messages.
+ */
+static const char SCIM_BRIDGE_MESSAGE_KANA_RO[] = "kana_ro";
+
 /**
  * The string constant of "unknown" for messages.
  */
