@@ -6,16 +6,15 @@
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.*
+ * License as published by the Free Software Foundation and 
+ * appearing in the file LICENSE.LGPL included in the package of this file.
+ * You can also redistribute it and/or modify it under the terms of 
+ * the GNU General Public License as published by the Free Software Foundation and 
+ * appearing in the file LICENSE.GPL included in the package of this file.
+ *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the
- * GNU Lesser General Public License for more details.*
- * You should have received a copy of the GNU Lesser General Public
- * License along with this program; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
- * Boston, MA  02111-1307  USA
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  */
 
 #include <assert.h>
@@ -57,7 +56,7 @@ struct _ScimBridgeMessenger
 /* Implementations */
 ScimBridgeMessenger *scim_bridge_alloc_messenger (int socket_fd)
 {
-    scim_bridge_pdebugln (5, "scim_bridge_alloc_messenger ()");
+    scim_bridge_pdebugln (4, "scim_bridge_alloc_messenger ()");
 
     if (socket_fd < 0) {
         scim_bridge_perrorln ("An invalid file descriptor is given at scim_bridge_alloc_messenger ()");
@@ -97,7 +96,7 @@ ScimBridgeMessenger *scim_bridge_alloc_messenger (int socket_fd)
 
 void scim_bridge_free_messenger (ScimBridgeMessenger *messenger)
 {
-    scim_bridge_pdebugln (5, "scim_bridge_free_messenger ()");
+    scim_bridge_pdebugln (4, "scim_bridge_free_messenger ()");
     if (messenger == NULL) return;
 
     scim_bridge_close_messenger (messenger);
@@ -111,7 +110,7 @@ void scim_bridge_free_messenger (ScimBridgeMessenger *messenger)
 
 retval_t scim_bridge_close_messenger (ScimBridgeMessenger *messenger)
 {
-    scim_bridge_pdebugln (5, "scim_bridge_close_messenger ()");
+    scim_bridge_pdebugln (4, "scim_bridge_close_messenger ()");
 
     if (messenger == NULL) {
         scim_bridge_perrorln ("The pointer given as a messenger is NULL");
@@ -141,7 +140,7 @@ boolean scim_bridge_messenger_is_closed (const ScimBridgeMessenger *messenger)
 
 int scim_bridge_messenger_get_socket_fd (const ScimBridgeMessenger *messenger)
 {
-    scim_bridge_pdebugln (5, "scim_bridge_messenger_get_socket_fd ()");
+    scim_bridge_pdebugln (4, "scim_bridge_messenger_get_socket_fd ()");
 
     if (messenger == NULL) {
         scim_bridge_perrorln ("The pointer given as a messenger is NULL");
@@ -154,7 +153,7 @@ int scim_bridge_messenger_get_socket_fd (const ScimBridgeMessenger *messenger)
 
 retval_t scim_bridge_messenger_push_message (ScimBridgeMessenger *messenger, const ScimBridgeMessage *message)
 {
-    scim_bridge_pdebugln (5, "scim_bridge_messenger_push_message ()");
+    scim_bridge_pdebugln (4, "scim_bridge_messenger_push_message ()");
 
     if (messenger == NULL) {
         scim_bridge_perrorln ("The pointer given as a messenger is NULL");
@@ -168,7 +167,7 @@ retval_t scim_bridge_messenger_push_message (ScimBridgeMessenger *messenger, con
 
     const ssize_t arg_count = (ssize_t) scim_bridge_message_get_argument_count (message);
 
-    scim_bridge_pdebug (5, "message:");
+    scim_bridge_pdebug (4, "message:");
     
     int i;
     for (i = -1; i < arg_count; ++i) {
@@ -179,7 +178,7 @@ retval_t scim_bridge_messenger_push_message (ScimBridgeMessenger *messenger, con
             str = scim_bridge_message_get_argument (message, i);
         }
 
-        scim_bridge_pdebug (5, " %s", str);
+        scim_bridge_pdebug (4, " %s", str);
 
         const size_t str_length = strlen (str);
 
@@ -237,7 +236,7 @@ retval_t scim_bridge_messenger_push_message (ScimBridgeMessenger *messenger, con
             }
         }
     }
-    scim_bridge_pdebug (5, "\n");
+    scim_bridge_pdebug (4, "\n");
 
     return RETVAL_SUCCEEDED;
 }
@@ -245,7 +244,7 @@ retval_t scim_bridge_messenger_push_message (ScimBridgeMessenger *messenger, con
 
 retval_t scim_bridge_messenger_poll_message (ScimBridgeMessenger *messenger, ScimBridgeMessage **message)
 {
-    scim_bridge_pdebugln (5, "scim_bridge_messenger_poll_message ()");
+    scim_bridge_pdebugln (4, "scim_bridge_messenger_poll_message ()");
 
     if (messenger == NULL) {
         scim_bridge_perrorln ("The pointer given as a messenger is NULL");
@@ -379,11 +378,11 @@ retval_t scim_bridge_messenger_send_message (ScimBridgeMessenger *messenger, con
     FD_SET (fd, &select_set);
 
     if (timeout != NULL) {
-        struct timeval pooling_timeout;
-        pooling_timeout.tv_sec = timeout->tv_sec;
-        pooling_timeout.tv_usec = timeout->tv_usec;
+        struct timeval polling_timeout;
+        polling_timeout.tv_sec = timeout->tv_sec;
+        polling_timeout.tv_usec = timeout->tv_usec;
 
-        if (select (fd + 1, NULL, &select_set, &select_set, &pooling_timeout) < 0) {
+        if (select (fd + 1, NULL, &select_set, &select_set, &polling_timeout) < 0) {
             if (errno == EINTR) {
                 scim_bridge_pdebugln (2, "An interruption occurred at scim_bridge_messenger_send_message ()");
                 return RETVAL_SUCCEEDED;
@@ -475,11 +474,11 @@ retval_t scim_bridge_messenger_receive_message (ScimBridgeMessenger *messenger,
     FD_SET (fd, &select_set);
 
     if (timeout != NULL) {
-        struct timeval pooling_timeout;
-        pooling_timeout.tv_sec = timeout->tv_sec;
-        pooling_timeout.tv_usec = timeout->tv_usec;
+        struct timeval polling_timeout;
+        polling_timeout.tv_sec = timeout->tv_sec;
+        polling_timeout.tv_usec = timeout->tv_usec;
 
-        if (select (fd + 1, &select_set, NULL, &select_set, &pooling_timeout) < 0) {
+        if (select (fd + 1, &select_set, NULL, &select_set, &polling_timeout) < 0) {
             if (errno == EINTR) {
                 scim_bridge_pdebugln (2, "An interruption occurred at scim_bridge_messenger_receive_message ()");
                 return RETVAL_SUCCEEDED;
@@ -544,7 +543,7 @@ retval_t scim_bridge_messenger_receive_message (ScimBridgeMessenger *messenger,
 
 ssize_t scim_bridge_messenger_get_sending_buffer_size (const ScimBridgeMessenger *messenger)
 {
-    scim_bridge_pdebugln (4, "scim_bridge_messenger_get_sending_buffer_size ()");
+    scim_bridge_pdebugln (3, "scim_bridge_messenger_get_sending_buffer_size ()");
 
     if (messenger == NULL) {
         scim_bridge_perrorln ("The pointer given as a messenger is NULL");
@@ -558,7 +557,7 @@ ssize_t scim_bridge_messenger_get_sending_buffer_size (const ScimBridgeMessenger
 
 ssize_t scim_bridge_messenger_get_receiving_buffer_size (const ScimBridgeMessenger *messenger)
 {
-    scim_bridge_pdebugln (4, "scim_bridge_messenger_get_receiving_buffer_size ()");
+    scim_bridge_pdebugln (3, "scim_bridge_messenger_get_receiving_buffer_size ()");
 
     if (messenger == NULL) {
         scim_bridge_perrorln ("The pointer given as a messenger is NULL");
